import os
import typing

import httpx

from . import MythicInfinityClientEnvironment
from .gen.base_client import MythicInfinityBaseClient, AsyncMythicInfinityBaseClient
from .streaming_tts_client import StreamingTTSClient, AsyncStreamingTTSClient


class MythicInfinityClient(MythicInfinityBaseClient):
    def __init__(self, *, base_url: typing.Optional[str] = os.getenv("MYTHICINFINITY_BASE_ENDPOINT"),
                 environment: MythicInfinityClientEnvironment = MythicInfinityClientEnvironment.PRODUCTION,
                 api_key: typing.Optional[
                     typing.Union[str, typing.Callable[[], str]]
                 ] = os.getenv("MYTHICINFINITY_API_KEY"), timeout: typing.Optional[float] = None,
                 follow_redirects: typing.Optional[bool] = True, httpx_client: typing.Optional[httpx.Client] = None):
        super().__init__(base_url=base_url, environment=environment, api_key=api_key, timeout=timeout,
                         follow_redirects=follow_redirects, httpx_client=httpx_client)

        self.tts = StreamingTTSClient(client_wrapper=self._client_wrapper)


class AsyncMythicInfinityClient(AsyncMythicInfinityBaseClient):
    def __init__(self, *, base_url: typing.Optional[str] = None,
                 environment: MythicInfinityClientEnvironment = MythicInfinityClientEnvironment.PRODUCTION,
                 api_key: typing.Optional[
                     typing.Union[str, typing.Callable[[], str]]
                 ] = os.getenv("MYTHICINFINITY_API_KEY"), timeout: typing.Optional[float] = None,
                 follow_redirects: typing.Optional[bool] = True,
                 httpx_client: typing.Optional[httpx.AsyncClient] = None):
        super().__init__(base_url=base_url, environment=environment, api_key=api_key, timeout=timeout,
                         follow_redirects=follow_redirects, httpx_client=httpx_client)

        self.tts = AsyncStreamingTTSClient(client_wrapper=self._client_wrapper)



from .gen.types import (
    HttpValidationError,
    ValidationError,
    ValidationErrorLocItem,
    Voice,
    VoiceOptions,
)
from .gen.errors import UnprocessableEntityError
from .gen import tts
from .gen.environment import MythicInfinityClientEnvironment

from .client import AsyncMythicInfinityClient, MythicInfinityClient

__all__ = [
    "AsyncMythicInfinityClient",
    "HttpValidationError",
    "MythicInfinityClient",
    "MythicInfinityClientEnvironment",
    "UnprocessableEntityError",
    "ValidationError",
    "ValidationErrorLocItem",
    "Voice",
    "VoiceOptions",
    "tts",
]


# This file was auto-generated by Fern from our API Definition.

from .http_validation_error import HttpValidationError
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem
from .voice import Voice
from .voice_options import VoiceOptions

__all__ = [
    "HttpValidationError",
    "ValidationError",
    "ValidationErrorLocItem",
    "Voice",
    "VoiceOptions",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .voices.client import VoicesClient
from ..types.voice_options import VoiceOptions
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper
from .voices.client import AsyncVoicesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TtsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.voices = VoicesClient(client_wrapper=self._client_wrapper)

    def generate_internal(
        self,
        *,
        model_id: str,
        text: str,
        voice_id: str,
        voice_options: typing.Optional[VoiceOptions] = OMIT,
        format: typing.Optional[typing.Literal["wav_24000"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Primary text to speech endpoint.

        Parameters
        ----------
        model_id : str
            (required) The model ID to generate speech audio with.

        text : str
            (required) The desired text spoken in the generated audio.

        voice_id : str
            (required) The voice ID to use for generating audio.

        voice_options : typing.Optional[VoiceOptions]

        format : typing.Optional[typing.Literal["wav_24000"]]
            (optional) The audio format of the output audio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Yields
        ------
        typing.Iterator[bytes]
            Returns a stream of audio bytes.
        """
        with self._client_wrapper.httpx_client.stream(
            "api/v1/audio/text-to-speech",
            method="POST",
            json={
                "model_id": model_id,
                "text": text,
                "voice_id": voice_id,
                "voice_options": convert_and_respect_annotation_metadata(
                    object_=voice_options, annotation=VoiceOptions, direction="write"
                ),
                "format": format,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:
            try:
                if 200 <= _response.status_code < 300:
                    _chunk_size = (
                        request_options.get("chunk_size", None)
                        if request_options is not None
                        else None
                    )
                    for _chunk in _response.iter_bytes(chunk_size=_chunk_size):
                        yield _chunk
                    return
                _response.read()
                if _response.status_code == 422:
                    raise UnprocessableEntityError(
                        typing.cast(
                            HttpValidationError,
                            parse_obj_as(
                                type_=HttpValidationError,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTtsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.voices = AsyncVoicesClient(client_wrapper=self._client_wrapper)

    async def generate_internal(
        self,
        *,
        model_id: str,
        text: str,
        voice_id: str,
        voice_options: typing.Optional[VoiceOptions] = OMIT,
        format: typing.Optional[typing.Literal["wav_24000"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Primary text to speech endpoint.

        Parameters
        ----------
        model_id : str
            (required) The model ID to generate speech audio with.

        text : str
            (required) The desired text spoken in the generated audio.

        voice_id : str
            (required) The voice ID to use for generating audio.

        voice_options : typing.Optional[VoiceOptions]

        format : typing.Optional[typing.Literal["wav_24000"]]
            (optional) The audio format of the output audio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Yields
        ------
        typing.AsyncIterator[bytes]
            Returns a stream of audio bytes.
        """
        async with self._client_wrapper.httpx_client.stream(
            "api/v1/audio/text-to-speech",
            method="POST",
            json={
                "model_id": model_id,
                "text": text,
                "voice_id": voice_id,
                "voice_options": convert_and_respect_annotation_metadata(
                    object_=voice_options, annotation=VoiceOptions, direction="write"
                ),
                "format": format,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:
            try:
                if 200 <= _response.status_code < 300:
                    _chunk_size = (
                        request_options.get("chunk_size", None)
                        if request_options is not None
                        else None
                    )
                    async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size):
                        yield _chunk
                    return
                await _response.aread()
                if _response.status_code == 422:
                    raise UnprocessableEntityError(
                        typing.cast(
                            HttpValidationError,
                            parse_obj_as(
                                type_=HttpValidationError,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class VoiceOptions(UniversalBaseModel):
    intensity: typing.Optional[float] = pydantic.Field(default=None)
    """
    (optional) Adjustable intensity factor for the generated audio. The default value varies per voice.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Voice(UniversalBaseModel):
    name: str
    voice_id: str
    model_ids: typing.List[str] = pydantic.Field()
    """
    Model IDs that this voice is compatible with.
    """

    preview_urls_by_model_id: typing.Dict[str, str] = pydantic.Field()
    """
    Preview urls for this voice per model id.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import os
import sys
import unittest

import io
import pytest
import wave

file_dir = os.path.dirname(
    os.path.abspath(
        os.path.realpath(__file__)))


class MythicInfinityClientTestCases(unittest.IsolatedAsyncioTestCase):
    async def asyncSetUp(self) -> None:
        # These values are checked for presence here but are picked up automatically by the client.
        api_key = os.getenv("MYTHICINFINITY_API_KEY")
        if api_key is None:
            raise Exception("No API Key present in env.")

        endpoint = os.getenv("MYTHICINFINITY_BASE_ENDPOINT")
        if endpoint is None:
            raise Exception("No test endpoint present in env.")

        # Ensure that we can import the sdk package from the source tree
        sys.path.append(f'{file_dir}/../')

    def _assert_audio_bytes(self, audio_bytes):
        assert audio_bytes is not None
        assert isinstance(audio_bytes, bytes)
        assert len(audio_bytes) > 44, "expected at least more than 44 byte header"

        with io.BytesIO(audio_bytes) as wav_file:
            with wave.open(wav_file, 'rb') as wf:
                # Get WAV file parameters
                nchannels = wf.getnchannels()
                sampwidth = wf.getsampwidth()
                framerate = wf.getframerate()
                nframes = wf.getnframes()
                comptype = wf.getcomptype()
                compname = wf.getcompname()

                # Read the audio frames
                frames = wf.readframes(nframes)

                assert framerate == 24_000
                assert nchannels == 1
                assert sampwidth == 2

    def test_nonstreaming_inference_with_sync_client(self):
        from mythicinfinity import MythicInfinityClient

        # api_key, base_url are set via environment variable
        client = MythicInfinityClient()

        ## Test .generate(stream=False)
        audio_bytes = client.tts.generate("Mythic Infinity is awesome!", stream=False)
        self._assert_audio_bytes(audio_bytes)

    def test_streaming_inference_with_sync_client(self):
        from mythicinfinity import MythicInfinityClient

        # api_key, base_url are set via environment variable
        client = MythicInfinityClient()

        ## Test .generate(stream=True)
        audio_gen = client.tts.generate("Mythic Infinity is awesome!", stream=True)
        audio_bytes = b''.join(list(audio_gen))

        self._assert_audio_bytes(audio_bytes)

    @pytest.mark.asyncio
    async def test_nonstreaming_inference_with_async_client(self):
        from mythicinfinity import AsyncMythicInfinityClient

        # api_key, base_url are set via environment variable
        client = AsyncMythicInfinityClient()

        ## Test .generate(stream=False)
        audio_bytes = await client.tts.generate("Mythic Infinity is awesome!", stream=False)
        self._assert_audio_bytes(audio_bytes)

    @pytest.mark.asyncio
    async def test_streaming_inference_with_async_client(self):
        from mythicinfinity import AsyncMythicInfinityClient

        # api_key, base_url are set via environment variable
        client = AsyncMythicInfinityClient()

        ## Test .generate(stream=True)
        audio_gen = await client.tts.generate("Mythic Infinity is awesome!", stream=True)
        audio_bytes = b''.join([x async for x in audio_gen])

        self._assert_audio_bytes(audio_bytes)



from __future__ import absolute_import
from __future__ import print_function

import io
import os
import re
from os.path import dirname
from os.path import join

from setuptools import find_packages
from setuptools import setup

this_file_dir = os.path.dirname(
    os.path.abspath(
        os.path.realpath(__file__)))


def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ).read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__\s*=\s*['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


def read_requirements(path):
    return [
        line.strip()
        for line in read(path).split("\n")
        if not line.startswith(('"', "#", "-", "git+"))
    ]


setup(
    name='mythicinfinity',
    version=find_version(this_file_dir, "mythicinfinity", "__version__.py"),
    license='MIT',
    description='Mythic Infinity python client library.',

    long_description=read(os.path.join(this_file_dir, "README.md")),
    long_description_content_type='text/markdown',

    author='Mythic Infinity',

    url='https://www.mythicinfinity.com/',
    project_urls={
        'Repository': 'https://github.com/mythicinfinity/mythicinfinity-python',
    },

    install_requires=[
        "httpx >= 0.21.2",
        "pydantic >= 1.9.2",
        "pydantic-core >= 2.18.2,==2.*",
        "typing_extensions >= 4.0.0",
    ],
    extras_require={"test": [
        "pytest==8.3.4",
        "pytest-xdist==3.6.1",
        "pytest-asyncio==0.25.3",
    ]},

    python_requires=">=3.8",

    packages=find_packages(exclude=["test", ".github"]),
    zip_safe=False,

    include_package_data=True,

    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent",
        "Operating System :: POSIX",
        "Operating System :: MacOS",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Typing :: Typed",
        "License :: OSI Approved :: MIT License"
    ]
)



import typing

from . import VoiceOptions
from .gen.core import RequestOptions
from .gen.tts.client import TtsClient, OMIT, AsyncTtsClient


DEFAULT_MODEL_ID = "mythic_realtime_v1"
DEFAULT_VOICE_ID = "kiera"

class StreamingTTSClient(TtsClient):
    @typing.overload
    def generate(
        self,
        text: str,
        *,
        stream: typing.Literal[False],
        model_id: str=DEFAULT_MODEL_ID,
        voice_id: str=DEFAULT_VOICE_ID,
        voice_options: typing.Optional[VoiceOptions] = OMIT,
        format: typing.Optional[typing.Literal["wav_24000"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> bytes:
        """
        Primary text to speech endpoint.

        Parameters
        ----------
        model_id : str
            (required) The model ID to generate speech audio with.

        text : str
            (required) The desired text spoken in the generated audio.

        voice_id : str
            (required) The voice ID to use for generating audio.

        voice_options : typing.Optional[VoiceOptions]

        format : typing.Optional[typing.Literal["wav_24000"]]
            (optional) The audio format of the output audio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        ------
        bytes
            Returns the audio bytes.
        """
        ...

    @typing.overload
    def generate(
        self,
        text: str,
        *,
        stream: typing.Literal[True],
        model_id: str=DEFAULT_MODEL_ID,
        voice_id: str=DEFAULT_VOICE_ID,
        voice_options: typing.Optional[VoiceOptions] = OMIT,
        format: typing.Optional[typing.Literal["wav_24000"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Primary text to speech endpoint.

        Parameters
        ----------
        model_id : str
            (required) The model ID to generate speech audio with.

        text : str
            (required) The desired text spoken in the generated audio.

        voice_id : str
            (required) The voice ID to use for generating audio.

        voice_options : typing.Optional[VoiceOptions]

        format : typing.Optional[typing.Literal["wav_24000"]]
            (optional) The audio format of the output audio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Yields
        ------
        typing.Iterator[bytes]
            Returns a stream of audio bytes.
        """
        ...

    def generate(
        self,
        text: str,
        *,
        stream: bool = False,
        model_id: str=DEFAULT_MODEL_ID,
        voice_id: str=DEFAULT_VOICE_ID,
        voice_options: typing.Optional[VoiceOptions] = OMIT,
        format: typing.Optional[typing.Literal["wav_24000"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Union[bytes, typing.Iterator[bytes]]:
        result_generator = super().generate_internal(model_id=model_id, text=text, voice_id=voice_id, voice_options=voice_options,
                               format=format, request_options=request_options)

        if stream:
            return result_generator

        audio_bytes = b''.join(result_generator)

        return audio_bytes


class AsyncStreamingTTSClient(AsyncTtsClient):
    @typing.overload
    async def generate(
        self,
        text: str,
        *,
        stream: typing.Literal[False],
        model_id: str=DEFAULT_MODEL_ID,
        voice_id: str=DEFAULT_VOICE_ID,
        voice_options: typing.Optional[VoiceOptions] = OMIT,
        format: typing.Optional[typing.Literal["wav_24000"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> bytes:
        """
        Primary text to speech endpoint.

        Parameters
        ----------
        model_id : str
            (required) The model ID to generate speech audio with.

        text : str
            (required) The desired text spoken in the generated audio.

        voice_id : str
            (required) The voice ID to use for generating audio.

        voice_options : typing.Optional[VoiceOptions]

        format : typing.Optional[typing.Literal["wav_24000"]]
            (optional) The audio format of the output audio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        ------
        bytes
            Returns the audio bytes.
        """
        ...

    @typing.overload
    async def generate(
        self,
        text: str,
        *,
        stream: typing.Literal[True],
        model_id: str=DEFAULT_MODEL_ID,
        voice_id: str=DEFAULT_VOICE_ID,
        voice_options: typing.Optional[VoiceOptions] = OMIT,
        format: typing.Optional[typing.Literal["wav_24000"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Primary text to speech endpoint.

        Parameters
        ----------
        model_id : str
            (required) The model ID to generate speech audio with.

        text : str
            (required) The desired text spoken in the generated audio.

        voice_id : str
            (required) The voice ID to use for generating audio.

        voice_options : typing.Optional[VoiceOptions]

        format : typing.Optional[typing.Literal["wav_24000"]]
            (optional) The audio format of the output audio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Yields
        ------
        typing.AsyncIterator[bytes]
            Returns a stream of audio bytes.
        """
        ...

    async def generate(
        self,
        text: str,
        *,
        stream: bool = False,
        model_id: str=DEFAULT_MODEL_ID,
        voice_id: str=DEFAULT_VOICE_ID,
        voice_options: typing.Optional[VoiceOptions] = OMIT,
        format: typing.Optional[typing.Literal["wav_24000"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Union[bytes, typing.AsyncIterator[bytes]]:
        async_result_generator = super().generate_internal(model_id=model_id, text=text, voice_id=voice_id, voice_options=voice_options,
                               format=format, request_options=request_options)

        if stream:
            return async_result_generator

        audio_bytes = b''.join([chunk async for chunk in async_result_generator])

        return audio_bytes